from django import template

from wagtailtrans.conf import get_wagtailtrans_setting
from wagtailtrans.models import TranslatablePage

register = template.Library()


@register.simple_tag
def get_canonical_pages_for_delete(page):
    """Get the translations made for this page

    :param page: Page instance
    :return: queryset or False
    """
    page = page.specific
    if get_wagtailtrans_setting('SYNC_TREE') and getattr(page, 'language', False) and not page.canonical_page:
        return TranslatablePage.objects.filter(canonical_page=page)
    return False



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-30 13:10
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailtrans', '0004_sitelanguages'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sitelanguages',
            name='other_languages',
            field=models.ManyToManyField(blank=True, to='wagtailtrans.Language'),
        ),
    ]



from .conf import get_wagtailtrans_setting
from .models import Language, SiteLanguages


def get_languages_for_site(site):
    """Utility to retrieve available languages for provided site."""
    if get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
        site_languages = SiteLanguages.for_site(site)
        languages = [site_languages.default_language] + list(site_languages.other_languages.all())
    else:
        languages = list(Language.objects.live().order_by('position'))

    return languages



from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand, CommandError

from wagtailtrans.models import Language
from wagtailtrans.utils.language_switch import change_default_language


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--language', type=str)

    def handle(self, language, *args, **kwargs):
        if not language:
            raise CommandError("Missing --language argument")
        try:
            new_default = Language.objects.get(code=language)
        except ObjectDoesNotExist:
            raise CommandError("Language is not yet set in your site")

        current_default = Language.objects.default()
        if new_default == current_default:
            raise CommandError("Language {} is already default language".format(new_default))

        change_default_language(new_default)



from django import forms
from django.utils.translation import ugettext as _
from wagtail.core.models import Page

from wagtailtrans.models import TranslatablePage


class TranslationForm(forms.ModelForm):
    copy_from_canonical = forms.BooleanField(required=False)
    parent_page = forms.ModelChoiceField(queryset=TranslatablePage.objects.none())

    class Meta:
        model = TranslatablePage
        fields = ['copy_from_canonical', 'parent_page']

    def __init__(self, *args, **kwargs):
        self.language = kwargs.pop('language')
        super(TranslationForm, self).__init__(*args, **kwargs)
        self.fields['parent_page'].queryset = self.get_queryset()

        if self._page_has_required(kwargs.get('instance')):
            self.fields['copy_from_canonical'].initial = True
            self.fields['copy_from_canonical'].disabled = True
            self.fields['copy_from_canonical'].help_text = _(
                "All fields need to be copied because of some required fields")

    def get_queryset(self):
        site = self.instance.get_site()
        qs = TranslatablePage.objects.filter(language=self.language).exclude(id=self.instance.id)

        allowed_pages = [p.pk for p in qs.specific() if self.instance.can_move_to(p) and p.get_site() == site]

        qs = qs.filter(pk__in=allowed_pages)
        if not qs:
            return Page.objects.filter(id=site.root_page_id)
        return qs

    def _page_has_required(self, page):
        common_fields = set(TranslatablePage._meta.fields)
        specific_fields = set(page.specific._meta.fields) - common_fields
        required_fields = [f for f in specific_fields if not f.blank and not f.name.endswith('ptr')]

        return len(required_fields) > 0



from django.core.exceptions import ObjectDoesNotExist
from django.db.models.signals import m2m_changed, post_save, pre_delete
from wagtail.admin.signals import init_new_page
from wagtail.core.models import Site, get_page_models

from wagtailtrans.conf import get_wagtailtrans_setting
from wagtailtrans.models import Language, SiteLanguages, TranslatablePage
from wagtailtrans.permissions import (
    create_group_permissions, get_or_create_language_group)


def synchronize_trees(sender, instance, **kwargs):
    """synchronize the translation trees when
    a TranslatablePage is created.

    :param sender: Sender model
    :param instance: TranslatablePage instance
    :param kwargs: kwargs e.g. created

    """
    try:
        site = instance.get_site()
    except ObjectDoesNotExist:
        return

    if get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
        site_default = site.sitelanguages.default_language
        is_default_language = instance.language == site_default
        other_languages = site.sitelanguages.other_languages.all()
    else:
        is_default_language = instance.language.is_default
        other_languages = Language.objects.filter(is_default=False)

    if not kwargs.get('created') or not getattr(instance, 'language', False) or not is_default_language:
        return

    for lang in other_languages:
        instance.create_translation(language=lang, copy_fields=True)


def synchronize_deletions(sender, instance, **kwargs):
    """We use pre_delete because when sync is disabled the foreign_key on
    canonical pages on_delete is set_null.

    :param sender: Sender model
    :param instance: TranslatablePage Instance
    :param kwargs: kwargs

    """
    language = getattr(instance, 'language', False)
    if language and instance.is_canonical:
        instance.get_translations(only_live=False).delete()


def create_new_language_tree_for_site(site, language):
    """Create a new language tree for a specific site.

    :param site: The site for which a new tree wil be created
    :param language: The language in which the tree wil be created
    """
    site_pages = site.root_page.get_children().values_list('pk', flat=True)
    default_language = (
        site.sitelanguages.default_language
        if get_wagtailtrans_setting('LANGUAGES_PER_SITE')
        else Language.objects.default()
    )
    canonical_home_page = TranslatablePage.objects.filter(pk__in=site_pages, language=default_language).first()
    if not canonical_home_page:
        # no pages created yet.
        return
    descendants = canonical_home_page.get_descendants(inclusive=True)
    for child_page in descendants:
        child_page = child_page.specific
        if hasattr(child_page, 'language') and not child_page.has_translation(language):
            child_page.create_translation(language, copy_fields=True)


def create_new_language_tree(sender, instance, **kwargs):
    """Signal will catch creation of a new language
    If sync trees is enabled it will create a whole new tree with
    correlating language.

    :param sender: Sender model
    :param instance: Language instance
    :param kwargs: kwargs e.g. created

    """
    if not kwargs.get('created'):
        return

    for site in Site.objects.all():
        create_new_language_tree_for_site(site, instance)


def update_language_trees_for_site(sender, instance, action, pk_set, **kwargs):
    """Create a new language tree for a site if a new language is added to it..

    :param sender: Sender model
    :param instance: Language instance
    :param action: The type of change to the m2m field
    :param pk_set: Pks of the changed relations
    :param kwargs: kwargs e.g. created

    """
    if action == 'post_add':
        for language in Language.objects.filter(pk__in=pk_set):
            create_new_language_tree_for_site(instance.site, language)


def create_language_permissions_and_group(sender, instance, **kwargs):
    """Create a new `Translator` role with it's required permissions.

    :param sender: Sender model
    :param instance: Language instance
    :param kwargs: kwargs e.g. created

    """
    if not kwargs.get('created'):
        return

    group = get_or_create_language_group(instance)
    create_group_permissions(group, instance)


def force_parent_language(**kwargs):
    """Force the initial language of the first page, before creating..

    When adding a homepage to a site, the initial language should be set.
    By default we set the default language from the Languages model, however
    when the languages are defined per site, it's possible that the default
    language differs from the database default.

    """
    page = kwargs.get('page')
    parent = kwargs.get('parent')

    #: Force the page language according to the parent, when the parent
    #: has no language set and is a site root page, force the default language
    #: For now we assume there isn't more than 1 site rooted at the parent.
    if hasattr(parent, 'language'):
        page.language = parent.language
    elif get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
        site = parent.sites_rooted_here.first()
        if site:
            lang_settings = SiteLanguages.for_site(site)
            page.language = lang_settings.default_language or Language.objects.default()


def register_signal_handlers():
    """Registers signal handlers.

    To create a signal for TranslatablePage we have to use wagtails
    get_page_model.

    """
    post_save.connect(create_language_permissions_and_group, sender=Language)
    init_new_page.connect(force_parent_language)
    if get_wagtailtrans_setting('SYNC_TREE'):
        if get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
            m2m_changed.connect(update_language_trees_for_site, sender=SiteLanguages.other_languages.through)
        else:
            post_save.connect(create_new_language_tree, sender=Language)

        for model in get_page_models():
            if hasattr(model, 'create_translation'):
                post_save.connect(synchronize_trees, sender=model)

            if hasattr(model, 'get_translations'):
                pre_delete.connect(synchronize_deletions, sender=model)



# Generated by Django 2.0.3 on 2018-03-27 09:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailtrans', '0006_auto_20161212_2020'),
    ]

    operations = [
        migrations.AlterField(
            model_name='language',
            name='is_default',
            field=models.BooleanField(default=False, help_text='Visitors with no language preference will see the site in this language'),
        ),
        migrations.AlterField(
            model_name='language',
            name='position',
            field=models.IntegerField(default=0, help_text='Language choices and translations will be displayed in this order'),
        ),
        migrations.AlterField(
            model_name='sitelanguages',
            name='default_language',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='site_default_language', to='wagtailtrans.Language'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-21 12:19
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0029_unicode_slugfield_dj19'),
        ('wagtailtrans', '0003_auto_20161121_1211'),
    ]

    operations = [
        migrations.CreateModel(
            name='SiteLanguages',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('default_language', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='site_default_language', to='wagtailtrans.Language')),
                ('other_languages', models.ManyToManyField(to='wagtailtrans.Language')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
            options={
                'verbose_name': 'Site languages',
                'verbose_name_plural': 'Site languages',
            },
        ),
    ]



default_app_config = 'wagtailtrans.apps.WagtailTransConfig'

VERSION = (2, 0, 3, 'final')


def get_version():
    """Return normalised version string."""
    version = '%s.%s' % (VERSION[0], VERSION[1])
    # Append 3rd digit if > 0
    if VERSION[2]:
        version = '%s.%s' % (version, VERSION[2])

    if VERSION[3] != 'final':
        version = '%s.%s' % (version, VERSION[3])
    return version



from django.conf.urls import url

from wagtailtrans.views.translation import TranslationView

app_name = 'wagtailtrans'

urlpatterns = [
    url(r'^(?P<instance_id>\d+)/add/(?P<language_code>[^/]+)/$', TranslationView.as_view(), name='add'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-12-12 20:20
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0029_unicode_slugfield_dj19'),
        ('wagtailtrans', '0005_auto_20161130_1310'),
    ]

    operations = [
        migrations.RenameField(
            model_name='translatablepage',
            old_name='page_ptr',
            new_name='translatable_page_ptr',
        ),
        migrations.AlterField(
            model_name='translatablepage',
            name='translatable_page_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='wagtailcore.Page'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-21 12:11
from django.db import migrations, models
import django.db.models.deletion
import wagtailtrans.models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailtrans', '0002_auto_20161106_1231'),
    ]

    operations = [
        migrations.AlterField(
            model_name='translatablepage',
            name='language',
            field=models.ForeignKey(default=wagtailtrans.models._language_default, on_delete=django.db.models.deletion.PROTECT, related_name='pages', to='wagtailtrans.Language'),
        ),
    ]



import logging

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

logger = logging.getLogger()


class WagtailTransConfig(AppConfig):
    label = 'wagtailtrans'
    name = 'wagtailtrans'
    verbose_name = _("Wagtail Translations")

    def ready(self):
        from wagtailtrans.signals import register_signal_handlers
        register_signal_handlers()



from django.db import models

from .conf import get_wagtailtrans_setting


class LanguageManager(models.Manager):
    """Custom manager for the `Language` model."""

    def live(self):
        """Return all the live languages."""
        return self.filter(live=True)

    def default(self):
        """Return the first choice of default languages."""
        return self.live().filter(is_default=True).first()

    def default_for_site(self, site):
        """Return default language for site"""
        if get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
            return self.filter(site_default_language__site=site).first()
        return self.default()



from django import forms
from django.template import Context, Template
from django.utils.safestring import mark_safe


class ReadOnlyWidget(forms.Select):
    """
    Render the original field as a hidden widget.
    And create a text display for the label
    """
    def __init__(self, text_display, *args, **kwargs):
        self.text_display = text_display
        self.initial_widget = forms.HiddenInput()
        super(ReadOnlyWidget, self).__init__(*args, **kwargs)

    def render(self, *args, **kwargs):
        original_content = self.initial_widget.render(*args, **kwargs)

        return mark_safe("""<span class="hidden">%s</span>%s""" % (original_content, self.text_display))


class CanonicalPageWidget(forms.Select):
    """Add a link to the canonical page, for ease of use."""

    def __init__(self, canonical_page, *args, **kwargs):
        self.canonical_page = canonical_page
        self.initial_widget = forms.HiddenInput()
        super(CanonicalPageWidget, self).__init__(*args, **kwargs)

    def render(self, *args, **kwargs):
        original_content = self.initial_widget.render(*args, **kwargs)
        template = Template("""
            {% load i18n %}
            <span class="hidden">{{ original_content }}</span>

            {% if canonical_page %}
                <a href="{% url 'wagtailadmin_pages:edit' canonical_page.id %}">{{ canonical_page.title }}</a>
            {% else %}
                {% trans "None" %}
            {% endif %}
        """)
        return mark_safe(template.render(Context({
            'canonical_page': self.canonical_page,
            'original_content': original_content,
        })))



from collections import OrderedDict

from django.template import Library

from wagtailtrans.sites import get_languages_for_site

register = Library()


def _get_translations(page, homepage_fallback=True, include_self=True):
    """
    Return URLs for translations of provided pages, if
    a language isn't available as direct translation

    Note: after https://github.com/wagtail/wagtail/pull/3354 gets merged
          we should update the URL functions for better perfomance.

    """
    site = page.get_site()
    available_translations = {}
    available_languages = get_languages_for_site(site)
    if hasattr(page, 'language'):
        if not include_self:
            available_languages.remove(page.language)

        page_translations = page.get_translations(only_live=True, include_self=include_self)
        available_translations = {p.language.code: p for p in page_translations}

    available_homepages = {}
    if homepage_fallback:
        available_homepages = {p.language.code: p for p in site.root_page.get_children().live().specific()}

    translations = OrderedDict()
    for language in available_languages:
        translation = available_translations.get(language.code)
        if translation:
            translations[language] = translation
        elif homepage_fallback:
            homepage = available_homepages.get(language.code)
            if homepage:
                translations[language] = homepage

    return translations


@register.simple_tag
def get_translations(page, homepage_fallback=True, include_self=True):
    """Return URLs for translations of the provided page.

    Usage:
        {% get_language_urls page as language_urls %}
        {% get_language_urls page homepage_fallback=False as language_urls %}
        {% get_language_urls page homepage_fallback=False include_self=False as language_urls %}  # noqa

    """
    return _get_translations(page, homepage_fallback=homepage_fallback, include_self=include_self)


@register.inclusion_tag('wagtailtrans/templatetags/language_selector.html')
def render_language_selector(page, homepage_fallback=True, include_self=False):
    """Render language selector template with the required context.

    Usage:
        {% render_language_selector page %}
        {% render_language_selector page homepage_fallback=False %}
        {% render_language_selector page homepage_fallback=False include_self=True %}  # noqa

    """
    available_translations = _get_translations(page, homepage_fallback=homepage_fallback, include_self=include_self)

    return {
        'current_page': page,
        'translations': available_translations,
    }



from django.conf import settings
from django.utils import translation
from django.utils.deprecation import MiddlewareMixin
from django.utils.translation import LANGUAGE_SESSION_KEY, check_for_language, get_language_from_path
from django.utils.translation.trans_real import get_languages, get_supported_language_variant

from .models import Language
from .sites import get_languages_for_site


def get_language_from_request(request):
    """
    Analyze the request to find what language the user wants the system to
    show. Only languages listed in settings.LANGUAGES are taken into account.
    If the user requests a sublanguage where we have a main language, we send
    out the main language.

    The URL path prefix will be checked for a language code.
    """
    lang_code = get_language_from_path(request.path_info)
    if lang_code is not None:
        return lang_code

    supported_lang_codes = get_languages()

    if hasattr(request, 'session'):
        lang_code = request.session.get(LANGUAGE_SESSION_KEY)
        if lang_code in supported_lang_codes and lang_code is not None and check_for_language(lang_code):
            return lang_code

    lang_code = request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME)

    try:
        return get_supported_language_variant(lang_code)
    except LookupError:
        return None


class TranslationMiddleware(MiddlewareMixin):
    def process_request(self, request):
        active_language = None

        language_from_request = get_language_from_request(request)
        requested_languages = request.META.get('HTTP_ACCEPT_LANGUAGE')
        if language_from_request:
            active_language = language_from_request
        elif requested_languages:
            requested_languages = requested_languages.split(',')
            codes = tuple(language.code for language in get_languages_for_site(request.site) if language)

            for language in requested_languages:
                language = language.split(';')[0]
                active_language = language if language in codes else None

                if active_language is None and language.startswith(codes):
                    active_language = [c for c in codes if language.startswith(c)][0]
                if active_language is not None:
                    break

        if active_language is None:
            default_language = Language.objects.default_for_site(site=request.site)

            if default_language:
                active_language = default_language.code
            else:
                active_language = settings.LANGUAGE_CODE

        translation.activate(active_language)
        request.LANGUAGE_CODE = active_language

    def process_response(self, request, response):
        if 'Content-Language' not in response:
            response['Content-Language'] = request.LANGUAGE_CODE

        response.set_cookie(
            settings.LANGUAGE_COOKIE_NAME, request.LANGUAGE_CODE,
            max_age=settings.LANGUAGE_COOKIE_AGE,
            path=settings.LANGUAGE_COOKIE_PATH,
            domain=settings.LANGUAGE_COOKIE_DOMAIN,
        )
        return response



from operator import itemgetter

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Q
from django.http import Http404
from django.shortcuts import redirect
from django.utils.encoding import force_text
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import activate
from wagtail.contrib.settings.models import BaseSetting
from wagtail.contrib.settings.registry import register_setting
from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel, PageChooserPanel
from wagtail.admin.forms import WagtailAdminModelForm, WagtailAdminPageForm
from wagtail.core.models import Page
from wagtail.search.index import FilterField

from .conf import get_wagtailtrans_setting
from .edit_handlers import CanonicalPageWidget, ReadOnlyWidget
from .managers import LanguageManager
from .permissions import TranslatableUserPagePermissionsProxy


class WagtailAdminLanguageForm(WagtailAdminModelForm):
    """Custom wagtailadmin form so we can make use of the panels
    property, used by ``wagtail.contrib.modeladmin``.

    """
    code = forms.ChoiceField(
        label=_("Language"), choices=settings.LANGUAGES,
        help_text=_("One of the languages defined in LANGUAGES"))

    class Meta:
        fields = [
            'code',
            'is_default',
            'position',
            'live',
        ]

    def __init__(self, *args, **kwargs):
        super(WagtailAdminLanguageForm, self).__init__(*args, **kwargs)

        sorted_choices = sorted(self.fields['code'].choices, key=itemgetter(1))
        self.fields['code'].choices = sorted_choices

    def clean_is_default(self):
        is_default = self.cleaned_data['is_default']

        if self.initial.get('is_default') and not is_default:
            raise ValidationError(_(
                "You can not remove is_default from a language. To change the "
                "default language, select is_default on a different language"))

        return is_default

    def save(self, commit=True):
        is_default = self.cleaned_data.get('is_default', False)
        if (
            not self.initial.get('is_default') == is_default and
            is_default and
            not get_wagtailtrans_setting('LANGUAGES_PER_SITE')
        ):
            from wagtailtrans.utils.language_switch import change_default_language  # noqa
            change_default_language(self.instance)
        return super(WagtailAdminLanguageForm, self).save(commit=commit)


def get_language_panels():
    children = [
        FieldPanel('code'),
        FieldPanel('position'),
        FieldPanel('live'),
    ]

    if not get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
        children.insert(1, FieldPanel('is_default'))

    return [
        MultiFieldPanel(heading=_("Language details"), children=children),
    ]


class Language(models.Model):
    """User defined language."""
    code = models.CharField(max_length=12, unique=True)

    is_default = models.BooleanField(
        default=False, help_text="""Visitors with no language preference will see the site in this language""")

    position = models.IntegerField(
        default=0, help_text="""Language choices and translations will be displayed in this order""")

    live = models.BooleanField(default=True, help_text="Is this language available for visitors to view?")

    objects = LanguageManager()

    base_form_class = WagtailAdminLanguageForm
    panels = get_language_panels()

    class Meta:
        ordering = ['position']

    def __str__(self):
        return force_text(dict(settings.LANGUAGES).get(self.code))

    def has_pages_in_site(self, site):
        return self.pages.filter(path__startswith=site.root_page.path).exists()


class AdminTranslatablePageForm(WagtailAdminPageForm):
    """Form to be used in the wagtail admin."""

    def __init__(self, *args, **kwargs):
        super(AdminTranslatablePageForm, self).__init__(*args, **kwargs)

        self.fields['canonical_page'].widget = CanonicalPageWidget(
            canonical_page=self.instance.specific.canonical_page)

        language_display = Language.objects.filter(pk=self.initial['language']).first()
        if self.instance.specific.is_canonical and language_display:
            language_display = "{} - {}".format(language_display, "canonical")

        self.fields['language'].widget = ReadOnlyWidget(text_display=language_display if language_display else '')


def _language_default():
    # Let the default return a PK, so migrations can also work with this value.
    # The FakeORM model in the migrations differ from this Django model.
    default_language = Language.objects.default()
    if default_language is None:
        return None
    else:
        return default_language.pk


class TranslatablePage(Page):

    #: Defined with a unique name, to prevent field clashes..
    translatable_page_ptr = models.OneToOneField(Page, parent_link=True, related_name='+', on_delete=models.CASCADE)
    canonical_page = models.ForeignKey(
        'self', related_name='translations', blank=True, null=True, on_delete=models.SET_NULL)
    language = models.ForeignKey(Language, related_name='pages', on_delete=models.PROTECT, default=_language_default)

    is_creatable = False

    search_fields = Page.search_fields + [
        FilterField('language_id'),
    ]

    settings_panels = Page.settings_panels + [
        MultiFieldPanel(
            heading=_("Translations"),
            children=[
                FieldPanel('language'),
                PageChooserPanel('canonical_page'),
            ]
        )
    ]

    base_form_class = AdminTranslatablePageForm

    def get_admin_display_title(self):
        return "{} ({})".format(super(TranslatablePage, self).get_admin_display_title(), self.language)

    def serve(self, request, *args, **kwargs):
        activate(self.language.code)
        return super(TranslatablePage, self).serve(request, *args, **kwargs)

    def move(self, target, pos=None, suppress_sync=False):
        """Move the page to another target.

        :param target: the new target to move the page to
        :param pos: position of the page in the new target
        :param suppress_sync: suppress syncing the translated pages

        """
        super(TranslatablePage, self).move(target, pos)

        if get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
            site = self.get_site()
            lang_settings = SiteLanguages.for_site(site)
            is_default = lang_settings.default_language == self.language
        else:
            is_default = self.language.is_default

        if not suppress_sync and get_wagtailtrans_setting('SYNC_TREE') and is_default:
            self.move_translated_pages(canonical_target=target, pos=pos)

    def move_translated_pages(self, canonical_target, pos=None):
        """Move only the translated pages of this instance (not self).

        This is only called when WAGTAILTRANS_SYNC_TREE is enabled

        :param canonical_target: Parent of the canonical page
        :param pos: position

        """
        translations = self.get_translations(only_live=False)
        if getattr(canonical_target, 'canonical_page', False):
            canonical_target = canonical_target.canonical_page

        for page in translations:
            # get target because at this point we assume the tree is in sync.
            target = TranslatablePage.objects.filter(
                Q(language=page.language),
                Q(canonical_page=canonical_target) | Q(pk=canonical_target.pk)
            ).get()

            page.move(target=target, pos=pos, suppress_sync=True)

    def get_translations(self, only_live=True, include_self=False):
        """Get all translations of this page.

        This page itself is not included in the result, all pages
        are sorted by the language position.

        :param only_live: Boolean to filter on live pages & languages.
        :return: TranslatablePage instance

        """
        canonical_page_id = self.canonical_page_id or self.pk
        translations = TranslatablePage.objects.filter(Q(canonical_page=canonical_page_id) | Q(pk=canonical_page_id))

        if not include_self:
            translations = translations.exclude(pk=self.pk)

        if only_live:
            translations = translations.live().filter(language__live=True)

        return translations

    def has_translation(self, language):
        """Check if page isn't already translated in given language.

        :param language: Language instance
        :return: Boolean

        """
        return language.pages.filter(canonical_page=self).exists()

    def get_translation_parent(self, language):
        site = self.get_site()
        if not language.has_pages_in_site(site):
            return site.root_page

        translation_parent = (
            TranslatablePage.objects
            .filter(canonical_page=self.get_parent(), language=language, path__startswith=site.root_page.path)
            .first()
        )
        return translation_parent

    def create_translation(self, language, copy_fields=False, parent=None):
        """Create a translation for this page. If tree syncing is enabled the
        copy will also be moved to the corresponding language tree.

        :param language: Language instance
        :param copy_fields: Boolean specifying if the content should be copied
        :param parent: Parent page instance for the translation
        :return: new Translated page (or subclass) instance

        """
        if self.has_translation(language):
            raise Exception("Translation already exists")

        if not parent:
            parent = self.get_translation_parent(language)

        if self.slug == self.language.code:
            slug = language.code
        else:
            slug = '%s-%s' % (self.slug, language.code)

        update_attrs = {
            'title': self.title,
            'slug': slug,
            'language': language,
            'live': False,
            'canonical_page': self,
        }

        if copy_fields:
            kwargs = {'update_attrs': update_attrs}
            if parent != self.get_parent():
                kwargs['to'] = parent

            new_page = self.copy(**kwargs)
        else:
            model_class = self.content_type.model_class()
            new_page = model_class(**update_attrs)
            parent.add_child(instance=new_page)

        return new_page

    @cached_property
    def has_translations(self):
        return self.translations.exists()

    @cached_property
    def is_canonical(self):
        return not self.canonical_page_id and self.has_translations


def get_user_language(request):
    """Get the Language corresponding to a request.
    return default language if Language does not exist in site

    :param request: Request object
    :return: Language instance
    """
    if hasattr(request, 'LANGUAGE_CODE'):
        language = Language.objects.live().filter(code=request.LANGUAGE_CODE).first()
        if language:
            return language
    return Language.objects.default_for_site(site=request.site)


class TranslatableSiteRootPage(Page):
    """Root page of any translatable site.

    This page should be used as the root page because it will
    route the requests to the right language.

    """
    parent_page_types = ['wagtailcore.Page']

    def serve(self, request, *args, **kwargs):
        """Serve TranslatablePage in the correct language

        :param request: request object
        :return: Http302 or Http404

        """
        language = get_user_language(request)
        candidates = TranslatablePage.objects.live().specific().child_of(self)
        try:
            translation = candidates.filter(language=language).get()
            return redirect(translation.url)
        except TranslatablePage.DoesNotExist:
            raise Http404


def page_permissions_for_user(self, user):
    """Patch for the page permissions adding our custom proxy

    Note: Since wagtail doesn't call this method on the
          specific page we need to patch the default page
          implementation for this.

    :param user: User instance
    :return: user permissions for page

    """
    user_perms = TranslatableUserPagePermissionsProxy(user)
    return user_perms.for_page(self)


Page.permissions_for_user = page_permissions_for_user


class SiteLanguagesForm(WagtailAdminModelForm):
    """Form to be used in the wagtail admin."""

    def save(self, commit=True):
        data = self.cleaned_data
        if not data['default_language'].pk == self.initial['default_language']:
            from wagtailtrans.utils.language_switch import change_default_language  # noqa
            change_default_language(data['default_language'], self.instance.site)

        return super(SiteLanguagesForm, self).save(commit=commit)


def register_site_languages():
    def decorate(func):
        if get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
            return register_setting(func)
        return func
    return decorate


@register_site_languages()
class SiteLanguages(BaseSetting):
    """Site specific settings are stored in the database"""
    default_language = models.ForeignKey(
        Language, related_name="site_default_language", null=True, on_delete=models.PROTECT)
    other_languages = models.ManyToManyField(Language, blank=True)

    panels = [
        MultiFieldPanel(
            heading=_("Languages"),
            children=[
                FieldPanel('default_language'),
                FieldPanel(
                    'other_languages', widget=forms.CheckboxSelectMultiple),
            ]
        ),
    ]

    base_form_class = SiteLanguagesForm

    class Meta:
        verbose_name = _("Site languages")
        verbose_name_plural = _("Site languages")



from django.urls import reverse
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext_lazy as _
from wagtail.admin.views.generic import CreateView

from wagtailtrans.forms import TranslationForm
from wagtailtrans.models import Language, TranslatablePage


class TranslationView(CreateView):
    model = TranslatablePage
    form_class = TranslationForm

    def page_title(self):
        return _("Translate {} to {}".format(self.instance.get_admin_display_title(), self.language))

    def get_add_url(self):
        return reverse('wagtailtrans_translations:add', args=(self.instance.id, self.language.code))

    def dispatch(self, request, instance_id, language_code, *args, **kwargs):
        self.language = get_object_or_404(Language, code=language_code)
        self.instance = get_object_or_404(TranslatablePage, id=instance_id).specific
        return super(TranslationView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(TranslationView, self).get_form_kwargs()
        kwargs['language'] = self.language
        kwargs['instance'] = self.instance
        return kwargs

    def form_valid(self, form):
        parent = form.cleaned_data['parent_page']
        copy_from_canonical = form.cleaned_data['copy_from_canonical']
        new_page = self.instance.create_translation(self.language, copy_fields=copy_from_canonical, parent=parent)
        return redirect('wagtailadmin_pages:edit', new_page.id)



from django.db import transaction
from django.db.models import Q

from wagtailtrans.models import Language, TranslatablePage


def create_new_canonical_page_mapping(new_language, queryset):
    return dict(queryset.filter(language_id=new_language.pk).values_list('canonical_page_id', 'pk'))


def get_page_queryset(site=None):
    if site is not None:
        pk_list = site.root_page.get_descendants().values_list('pk', flat=True)
        return TranslatablePage.objects.filter(pk__in=pk_list)

    return TranslatablePage.objects.all()


@transaction.atomic
def change_default_language(new_language, site=None):
    """Change the default language
    for the site or all sites if no site is given

    """
    if site is None:
        old_language = Language.objects.default()
        if old_language is not None:
            old_language.is_default = False
            old_language.save()

        new_language.is_default = True
        new_language.save()

    queryset = get_page_queryset(site)
    mapping = create_new_canonical_page_mapping(new_language, queryset)

    # New canonical languages should not have a canonical page
    queryset.filter(pk__in=mapping.values()).update(canonical_page_id=None)

    for old_pk, new_pk in mapping.items():
        # Set new canonical page id to the new canonical pages.
        queryset.filter(Q(canonical_page_id=old_pk) | Q(pk=old_pk)).update(canonical_page_id=new_pk)
        # new canonical pages can't have a canonical page
        # so we set the canonical_page_id to None
        queryset.filter(pk=new_pk).update(canonical_page_id=None)



import os
import sys

from setuptools import find_packages, setup

PROJECT_DIR = os.path.dirname(__file__)

sys.path.append(os.path.join(PROJECT_DIR, 'src'))
from wagtailtrans import get_version  # noqa isort:skip

sandbox_require = [
    'Django>=2.0',
    'Wagtail>=2.0',
    'psycopg2>=2.5.4',
    'djangorestframework>=3.7',
]

docs_require = [
    'sphinx',
    'sphinx_rtd_theme',
]

tests_require = [
    # Required for test and coverage
    'pytest',
    'pytest-cov',
    'pytest-django',
    'coverage',
    'factory-boy',
    'psycopg2>=2.5.4',
    # Linting
    'flake8',
    'isort',
]

setup(
    name='wagtailtrans',
    version=get_version().replace(' ', '-'),
    description='A Wagtail add-on for supporting multilingual sites',
    author='Lukkien BV',
    author_email='support@lukkien.com',
    url='https://lukkien.com/',
    extras_require={
        'test': tests_require,
        'doc': docs_require,
        'sandbox': sandbox_require + tests_require + docs_require,
    },
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    license='BSD',
    long_description=open('README.rst').read(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ]
)



from django.conf import settings

DEFAULT_SETTINGS = {
    'SYNC_TREE': True,
    'LANGUAGES_PER_SITE': False,
    'HIDE_TRANSLATION_TREES': False,
}


def get_wagtailtrans_setting(name):
    return getattr(settings, 'WAGTAILTRANS_{}'.format(name), DEFAULT_SETTINGS[name])



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-06 12:31
from django.db import migrations, models
import django.db.models.deletion
import wagtailtrans.models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailtrans', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='language',
            name='code',
            field=models.CharField(max_length=12, unique=True),
        ),
    ]



from django.conf.urls import include, url
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.urls import reverse
from django.utils.encoding import force_text
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from wagtail.admin import widgets
from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register
from wagtail.core import hooks
from wagtailtrans.conf import get_wagtailtrans_setting
from wagtailtrans.models import Language, TranslatablePage
from wagtailtrans.urls import translations


class LanguageModelAdmin(ModelAdmin):
    add_to_settings_menu = True
    model = Language
    menu_label = _("Languages")
    menu_icon = 'icon icon-wagtail'
    menu_order = 1000
    list_display = ['__str__', 'position', 'live', 'is_default']
    list_filter = ['live']


modeladmin_register(LanguageModelAdmin)


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^translate/', include(translations, namespace='wagtailtrans_translations')),
    ]


if get_wagtailtrans_setting('LANGUAGES_PER_SITE'):
    @hooks.register('insert_global_admin_js')
    def global_admin_js():
        return format_html(
            '<script type="text/javascript" src="{path}"></script>'.format(
                path=static('wagtailtrans/js/site_languages_editor.js'))
        )


if not get_wagtailtrans_setting('SYNC_TREE'):
    """Only load hooks when WAGTAILTRANS_SYNC_TREE is disabled"""

    @hooks.register('register_page_listing_buttons')
    def page_translations_menu(page, page_perms, is_parent=False):
        if not hasattr(page, 'language'):
            return

        if hasattr(page, 'canonical_page') and page.canonical_page:
            return

        yield widgets.ButtonWithDropdownFromHook(
            'Translate into',
            hook_name='wagtailtrans_dropdown_hook',
            page=page,
            page_perms=page_perms,
            is_parent=is_parent,
            priority=10
        )

    @hooks.register('wagtailtrans_dropdown_hook')
    def page_translations_menu_items(page, page_perms, is_parent=False):
        prio = 1
        exclude_lang = None

        if hasattr(page, 'language') and page.language:
            exclude_lang = page.language

        other_languages = set(Language.objects.live().exclude(pk=exclude_lang.pk).order_by('position'))

        translations = page.get_translations(only_live=False).select_related('language')
        taken_languages = set(t.language for t in translations)

        translation_targets = other_languages - taken_languages
        for language in translation_targets:
            yield widgets.Button(
                force_text(language),
                reverse('wagtailtrans_translations:add', kwargs={
                    'instance_id': page.pk,
                    'language_code': language.code,
                }),
                priority=prio)

            prio += 1


@hooks.register('construct_explorer_page_queryset')
def hide_non_canonical_languages(parent_page, pages, request):
    """Hide translations when WAGTAILTRANS_HIDE_TRANSLATION_TREES=True.

    This allows the user to only see the canonical language in the admin.

    """
    if parent_page.depth > 1 and get_wagtailtrans_setting('HIDE_TRANSLATION_TREES'):
        return pages.filter(
            pk__in=(
                TranslatablePage.objects
                .filter(canonical_page__isnull=True)
                .values_list('pk', flat=True)
            )
        )
    return pages


@hooks.register('register_page_listing_buttons')
def edit_in_language_button(page, page_perms, is_parent=False):
    """Add ``Edit in`` button to the page explorer.

    When hiding all other translation except the canonical language, which is
    done via ``WAGTAILTRANS_HIDE_TRANSLATION_TREES`` this will add an button to
    allow the user to select a other language to edit, which provides a more
    clear interface to work in.

    """
    if not hasattr(page, 'language'):
        return

    yield widgets.ButtonWithDropdownFromHook(
        _("Edit in"),
        hook_name='wagtailtrans_dropdown_edit_hook',
        page=page,
        page_perms=page_perms,
        is_parent=is_parent,
        priority=10
    )


@hooks.register('wagtailtrans_dropdown_edit_hook')
def edit_in_language_items(page, page_perms, is_parent=False):
    """Add all other languages in the ``Edit in`` dropdown.

    All languages other than the canonical language are listed as dropdown
    options which allows the user to click on them and edit the page in the
    language they prefer.

    """
    other_languages = (
        page.specific
        .get_translations(only_live=False)
        .exclude(pk=page.pk)
        .select_related('language')
        .order_by('language__position')
    )

    for prio, language_page in enumerate(other_languages):
        yield widgets.Button(
            force_text(language_page.language),
            reverse('wagtailadmin_pages:edit', args=(language_page.pk,)),
            priority=prio,
        )
